--[[--------------------------------------------------------------------------------
CooldownTimers -  Main File
Version: 2.$Revision: 10808 $
Author: Krsnik
Credits: Sole for HostelBar where I got most of the stuff I used, Ammo for CandyBar :) and of course the Ace community
----------------------------------------------------------------------------------]]
local defaults = {
	-- general opts
	mintime = 2,			-- min time to show a bar
	maxtime = 1800,			-- max time to show a bar
	bgalpha = 1, 			-- background opacity of the bars, ranges from 0.0 to 1.0 
	bargrowth = false,		-- direction the bars will grow in,  true = up, false = down
	barcolor = "green",		-- possible values: white, black, blue, green, yellow, orange, red, magenta, cyan
	bgcolor = "cyan",		-- possible values: white, black, blue, green, yellow, orange, red, magenta, cyan
	textcolor = "white",	-- possible values: white, black, blue, green, yellow, orange, red, magenta, cyan
	barscale = 1.0,			-- size of the bars
	barheight = 12,			-- height of the bars
	barwidth = 150,			-- width of the bars
	position = {},			-- stored position
	ignore = {}				-- ignored spells
}

local resetSpells = {
	["Preparation"] = {
		[1] = "Vanish",
		[2] = "Cold Blood",
		[3] = "Sprint",
		[4] = "Premeditation",
		[5] = "Blind",
		[6] = "Evasion",
		[7] = "Kick",
		[8] = "Kidney Shot",
		[9] = "Ghostly Strike",
		[10] = "Distract",
		[11] = "Feint",
	},
	["Cold Snap"] = {
		[1] = "Frost Nova",
		[2] = "Ice Barrier",
		[3] = "Cone of Cold",
		[4] = "Ice Block",
		[5] = "Frost Ward",
	},
}
--[[--------------------------------------------------------------------------------
  Class Setup
-----------------------------------------------------------------------------------]]
CooldownTimers = AceLibrary("AceAddon-2.0"):new("AceEvent-2.0", "AceDB-2.0", "AceConsole-2.0", "AceDebug-2.0")
-- embedded libs
local L = AceLibrary("AceLocale-2.0"):new("CooldownTimers")
local BS = AceLibrary("Babble-Spell-2.0")
local compost = AceLibrary("Compost-2.0")

function CooldownTimers:OnInitialize()
	--embedded libs
	self.candy = AceLibrary("CandyBar-2.0")
	self.gt = AceLibrary("Gratuity-2.0")
	self.def = AceLibrary("Deformat-2.0")

	-- set chat commands 
	local args = {
	type = "group",
	args = {
		ignore = {
			name = L["Ignore"], desc = L["DescIgnore"],	type = "group",	
			args = {
				add = {
					name = "Add",
					desc = "Add a spell to the ignore list.",
					usage = "<filter>",
					type = "text",
					get = false,
					set = function(v)
						self.db.profile.ignore[string.lower(v)] = true
						self:Print("Added spell |cff00ff00%s|r to the list", v)
					end,
				},
				remove = {
					name = "Remove",
					desc = "Remove a spell from the ignore list",
					usage = "<filter>",
					aliases = "rm, del",
					type = "text",
					get = false,
					set = function(v)
						if self.db.profile.ignore[string.lower(v)] then
							self.db.profile.ignore[string.lower(v)] = nil
							self:Print("Spell |cff00ff00%s|r removed", v)
						else
							self:Print("Spell |cff00ff00%s|r |cffff0000not found|r in the list", v)
						end
					end,
				},
				list = {
					name = "List filters",
					desc = "Output the list of spells being ignored",
					aliases = "ls",
					type = "execute",
					func = function()
						local emptyCheck
						for k in pairs(self.db.profile.ignore) do
							self:Print("|cff00ff00%s|r", k)
							emptyCheck = true
						end
						if not emptyCheck then self:Print("You have no active filters") end
					end,
				},
				reset = {
					name = "Reset filters",
					desc = "Empty the ignore list",
					type = "execute",
					func = function()
						self.db.profile.ignore = {}
						self:Print("The ignore list is now empty")
					end,
				},
			}
		},
		mintime = {
			name = L["Min Time"],desc = L["DescMinTime"],type = "range",
			get = function () return self.db.profile.mintime end,
			set = function (v) self.db.profile.mintime = v end,
			min = 1.0, max = 10,
		},
		maxtime = {
			name = L["Max Time"],desc = L["DescMaxTime"],type = "range",
			get = function () return self.db.profile.maxtime end,
			set = function (v) self.db.profile.maxtime = v end,
			min = 2.0, max = 3600,
		},
		bgalpha = {
			name = L["Background Alpha"],desc = L["DescBGAlpha"],type = "range",
			get = function () return self.db.profile.bgalpha end,
			set = function (v) self.db.profile.bgalpha = v end,
			min = 0.0, max = 1.0,
		},
		bargrowth = {
			name = L["Bar Growth"], desc=L["DescBarGrowth"], type = "toggle",
			get = function () return self.db.profile.bargrowth end,
			set = function (v) self.db.profile.bargrowth.Default = v  
				self.candy:SetCandyBarGroupGrowth("CDT", self.db.profile.bargrowth) end,
				map = { [false] = L["MapBarGrowthFalse"], [true] = L["MapBarGrowthTrue"],},
		},
		barcolor = {
			name = L["Bar Color"], desc=L["DescBarColor"], type = "text",
			get = function () return self.db.profile.barcolor end,
			set = function (v) self.db.profile.barcolor = v end, 
			validate = { "white", "black", "blue", "green", "yellow", "orange", "red", "magenta", "cyan" },
		},
		bgcolor = {
			name = L["Background Color"], desc=L["DescBgColor"], type = "text",
			get = function () return self.db.profile.bgcolor end,
			set = function (v) self.db.profile.bgcolor = v end, 
			validate = { "white", "black", "blue", "green", "yellow", "orange", "red", "magenta", "cyan" },
		},
		textcolor = {
			name = L["Text Color"], desc=L["DescTextColor"], type = "text",
			get = function () return self.db.profile.textcolor end,
			set = function (v) self.db.profile.textcolor = v end, 
			validate = { "white", "black", "blue", "green", "yellow", "orange", "red", "magenta", "cyan" },
		},
		barheight = {
			name = L["Bar Height"], desc=L["DescBarHeight"], type = "range",
			get = function () return self.db.profile.barheight end,
			set = function (v) self.db.profile.barheight = v end,
			min = 8, max = 30,
		},
		barwidth = {
			name = L["Bar Width"], desc=L["DescBarWidth"], type = "range",
			get = function () return self.db.profile.barwidth end,
			set = function (v) self.db.profile.barwidth = v end,
			min = 50, max = 300,
		},
		barscale = {
			name = L["Bar Scale"], desc=L["DescBarScale"], type = "range",
			get = function () return self.db.profile.barscale end,
			set = function (v) self.db.profile.barscale = v end,
			min = 0.5, max = 1.5,
		},
		anchor = {name = L["Show Anchor"], desc = L["DescShowAnchor"], type = "execute", func = "ToggleAnchor"},
		test = {name = L["Test"],desc = L["DescTest"],type = "execute",func = "RunTestTimer"},
	}
	}

    -- register stuff 
	CooldownTimers:RegisterDB("CooldownTimersDB")
	CooldownTimers:RegisterDefaults('profile', defaults)

	CooldownTimers:RegisterChatCommand({"/cdt"}, args)
	
	-- create the anchor frames	
	self.anchor = self:CreateAnchor(L["GUIAnchorString"], 0, 0,1,0)

	self:CreateCandyBarRegs()
end

function CooldownTimers:OnEnable()
	if( not self.spells ) then self.spells = {} end
	self:RefreshRegisteredEvents()
end
function CooldownTimers:OnDisable()
	if( self.spells) then self.spells = {} end
	-- no need to do self:UnregisterAllEvents() as Ace automatically does it when when an addon is disabled, so just unreg the parserlib events
	parser:UnregisterAllEvents("CooldownTimers")
end
--[[--------------------------------------------------------------------------------
  Event Registering
-----------------------------------------------------------------------------------]]
-- CandyBar group registrations
function CooldownTimers:CreateCandyBarRegs()
	self.candy:RegisterCandyBarGroup("CooldownTimers")
	self.candy:SetCandyBarGroupPoint("CooldownTimers", "TOP", self.anchor, "BOTTOM", 0, 0)	
	self.candy:SetCandyBarGroupGrowth("CooldownTimers", self.db.profile.bargrowth) 
end

-- Event registrations
function CooldownTimers:RefreshRegisteredEvents()
	--self:RegisterEvent("SPELL_UPDATE_COOLDOWN", "updateCooldowns")
	self:RegisterEvent("SPELLS_CHANGED", "parseSpells")
	self:RegisterEvent("PLAYER_ENTERING_WORLD", function() self:parseSpells() self:UnregisterEvent("PLAYER_ENTERING_WORLD") end)
	self:RegisterEvent("CHAT_MSG_SPELL_AURA_GONE_SELF", function (arg1) self:execStealth(arg1) end)
	
	-- SpellStatus Events
	self:RegisterEvent("SpellStatus_SpellCastInstant", function (spellId, spellName) self:ParseSpellStatus(spellId, spellName) end)
	self:RegisterEvent("SpellStatus_SpellCastCastingFinish", function (spellId, spellName) self:ParseSpellStatus(spellId, spellName) end)
	self:RegisterEvent("SpellStatus_SpellCastChannelingStart", function (spellId, spellName) self:ParseSpellStatus(spellId, spellName) end)

	self:Debug("RefreshRegisteredEvents: Events registered")
	regevent = nil
end
--[[--------------------------------------------------------------------------------
  Event Parsing
-----------------------------------------------------------------------------------]]

-- Parse SpellStatus events
function CooldownTimers:ParseSpellStatus(spellId, spellName)
	if not self.spells then return end
	self:Debug("SpellStatus: "..spellName)
		
	if self.spells[spellName] then
		self:Debug("Spell Found:"..self.spells[spellName])
		for k in resetSpells do
			if self:ReverseTranslateSpell(spellName) == k then self:ExecResetSpells(k) end
		end
		self:CheckInfo(spellName)	
	end
end

-- Parse SpellBook
function CooldownTimers:parseSpells()
	self:ParseSpellBook(BOOKTYPE_SPELL)
	self:ParseSpellBook(BOOKTYPE_PET)
end



--[[--------------------------------------------------------------------------------
  Utility Functions
-----------------------------------------------------------------------------------]]
function CooldownTimers:ParseSpellBook(type)
	local i, n, n2, r, cd = 1
	self:Debug("Running - parseSpells")
	while true do
		n, r = GetSpellName(i, type)
		n2 = GetSpellName(i+1, type)
		if not n then
			do break end
		end
		
		if(n ~= n2) then
			self.gt:SetSpell(i, type)			
			cd = self.gt:GetLine(3, 1) or self.gt:GetLine(2, 1)
			
			if(cd) then
				if(self.def(cd, SPELL_RECAST_TIME_SEC)) then
					self.spells[n] = self.def(cd, SPELL_RECAST_TIME_SEC)
					self:Debug("%s has %s sec cooldown on %s", n, self.def(cd, SPELL_RECAST_TIME_SEC), r)
				elseif(self.def(cd, SPELL_RECAST_TIME_MIN)) then
					self.spells[n] = self.def(cd, SPELL_RECAST_TIME_MIN)*60
					self:Debug("%s has %s sec cooldown on %s", n, self.def(cd, SPELL_RECAST_TIME_MIN)*60, r)
				end
			end
		end
		
		i = i + 1
	end
end

-- returns the icon of a spell
function CooldownTimers:GetSpellIcon(spell)
	local icon = BS:GetSpellIcon(spell)
	return icon
end

function CooldownTimers:ReverseTranslateSpell(spell)
	-- change spell to english if it isnt
	if GetLocale() ~= "enUS" or GetLocale() ~= "enGB" then	
		if BS:HasReverseTranslation(spell) then 
			spell = BS:GetReverseTranslation(spell) 
		elseif L:HasReverseTranslation("Spell"..spell) then
			spell = L:GetReverseTranslation("Spell"..spell)
		else
			self:Debug("ReverseTranslateSpell: no reverse translation for spell: "..spell)
		end 
	end
	return spell
end

-- returns translation of a spell if it exists
function CooldownTimers:TranslateSpell(spell)
	-- if the locale isnt english then translate the spell back to the original locale (we're now done with parsing)
	if GetLocale() ~= "enUS" or GetLocale() ~= "enGB" then	
		if BS:HasTranslation(spell) then
			spell = BS:GetTranslation(spell) 
		elseif L:HasTranslation("Spell"..spell) then
			spell = L:GetTranslation("Spell"..spell)
		else
			self:Debug("TranslateSpell: no translation for spell: "..spell)
		end
	end
	return spell
end

--[[--------------------------------------------------------------------------------
  Main Processing
-----------------------------------------------------------------------------------]]
function CooldownTimers:ExecResetSpells(spell)
	self:Debug( spell.." used!" )	
	if not resetSpells[spell] then return end
	
	for k,v in resetSpells[spell] do		
		if self.candy:IsCandyBarRegistered("CooldownTimers_"..self:TranslateSpell(v)) then 
			self:Debug( "Reset Skill used! Killing Bar: "..self:TranslateSpell(v))
			self.candy:StopCandyBar("CooldownTimers_"..self:TranslateSpell(v)) 
		end
	end
end

function CooldownTimers:execStealth(msg)
	local stealth = {
		[1] = "Stealth",
		[2] = "Shadowmeld",
		[3] = "Prowl"
	}
	
	for k,v in stealth do 
		if string.find(msg, v) then
			if self.candy:IsCandyBarRegistered("CooldownTimers_"..self:TranslateSpell(v)) then
				self:Debug( self:TranslateSpell(v).."faded! Killing Bar.")
				self.candy:StopCandyBar("CooldownTimers_"..self:TranslateSpell(v)) 
			end
		end
	end
end

function CooldownTimers:CheckInfo(spell)
	-- is this spell being ignored?
	if self.db.profile.ignore[string.lower(spell)] then self:Debug( "StartTimer: "..spell.." ignored") return end
	
	-- is it already running? excluding Durations bars
	-- if self.candy:IsCandyBarRegistered("CooldownTimers_"..spell) then self:Debug( "StartTimer: Bar "..spell.." already running") return end

	-- check if spell exists
	if not self.spells[spell] then return end
	
	-- Duration
	local time = self.spells[spell]
	
	if time >= self.db.profile.mintime and time <= self.db.profile.maxtime then
		-- passed all checks, on to the formatting!
		self:FormatBar(spell, time)
	end
end

-- Formats the bar's appearance
function CooldownTimers:FormatBar(spell, time)
	
	-- apply the colors
	local textcolor = self.db.profile.textcolor
	local bgcolor = self.db.profile.bgcolor
	local barcolor = self.db.profile.barcolor
	
	-- set alpha
	local bgalpha = self.db.profile.bgalpha
	
	-- set size
	local barwidth = self.db.profile.barwidth
	local barheight = self.db.profile.barheight
	
	local barscale = self.db.profile.barscale
	
	-- set the texture 
	local texture = self:GetSpellIcon(spell)
	
	-- formatting complete, start the bar 
	self:StartBar(spell, time, texture, barcolor, textcolor, bgcolor, bgalpha, barwidth, barheight, barscale)

end

-- Register and start the timer
function CooldownTimers:StartBar(spell, time, texture, barcolor, textcolor, bgcolor, bgalpha, barwidth, barheight, barscale)

	-- declare id var for ease of use 
	local id = "CooldownTimers_"..spell

	-- register the bar
	self.candy:RegisterCandyBar(id, time, spell, texture, barcolor, barcolor, "orange", "red")
	
	self.candy:SetCandyBarTextColor(id, textcolor)
	self.candy:SetCandyBarBackgroundColor(id, bgcolor, bgalpha)
	self.candy:SetWidth(id, barwidth)
	self.candy:SetHeight(id, barheight)
	self.candy:SetCandyBarScale(id, barscale)
	
	self.candy:SetCandyBarFade(id, .5) 

	-- register bar with group
	self.candy:RegisterCandyBarWithGroup(id, "CooldownTimers")
	
	-- start this bitch!
	self.candy:StartCandyBar(id, true)
	
	self:Debug("StartBar: Bar started [ Spell: "..spell..", Time: "..time.."]")

	-- reset variables
	id, spell, time, texture, barcolor, textcolor, bgcolor = nil
end

--[[--------------------------------------------------------------------------------
  Command Handlers
-----------------------------------------------------------------------------------]]
-- Runs test bars
function CooldownTimers:RunTestTimer()
	self.candy:RegisterCandyBar("CooldownTimers_Test", 10, L["GUITestBarText"], "Interface\\Icons\\Spell_Fire_Fireball02", "green", "green","red")
	self.candy:RegisterCandyBarWithGroup("CooldownTimers_Test", "CooldownTimers")
	
	self.candy:SetWidth("CooldownTimers_Test", self.db.profile.barwidth)
	self.candy:SetHeight("CooldownTimers_Test", self.db.profile.barheight)
	self.candy:SetCandyBarScale("CooldownTimers_Test", self.db.profile.barscale)
		
	self.candy:StartCandyBar("CooldownTimers_Test", true)
	
	self:Debug("RunTestTimer: test bar started")
end

-- Toggles the vis of the anchors
function CooldownTimers:ToggleAnchor()
	if self.anchor:IsVisible() then
		self.anchor:Hide()
	else
		self.anchor:Show()
	end
end

--[[--------------------------------------------------------------------------------
  GUI Constructors
-----------------------------------------------------------------------------------]]

-- Creates the anchor frames
function CooldownTimers:CreateAnchor(text, yoff, cRed, cGreen, cBlue)
	local f = CreateFrame("Button", nil, UIParent)
	f:ClearAllPoints()
	f:SetWidth(200)
	f:SetHeight(25)
	
	f.owner = self
	
	if self.db.profile.position.x and self.db.profile.position.y then
		f:SetPoint("BOTTOMLEFT", UIParent, "BOTTOMLEFT", self.db.profile.position.x, self.db.profile.position.y)
	else
		f:SetPoint("TOP", UIErrorsFrame, "BOTTOM", 0, yoff)
	end

	f:SetScript("OnDragStart", function() this:StartMoving() end )
	f:SetScript("OnDragStop",
		function()
			this:StopMovingOrSizing()
			local s = this:GetEffectiveScale()
			local x = math.floor(s * this:GetLeft())
			local y = math.floor(s * this:GetTop())
			this.owner.db.profile.position.x = x
			this.owner.db.profile.position.y = y
        end)


	f:SetBackdrop({bgFile = "Interface/Tooltips/UI-Tooltip-Background",
                                            edgeFile = "Interface/Tooltips/UI-Tooltip-Border",
                                            tile = false, tileSize = 16, edgeSize = 16,
                                            insets = { left = 5, right =5, top = 5, bottom = 5 }})
	f:SetBackdropColor(cRed,cGreen,cBlue,.6)
	f:SetMovable(true)
	f:RegisterForDrag("LeftButton")

	f.Text = f:CreateFontString(nil, "OVERLAY")
	f.Text:SetFontObject(GameFontNormalSmall)
	f.Text:ClearAllPoints()
	f.Text:SetTextColor(1, 1, 1, 1)
	f.Text:SetWidth(200)
	f.Text:SetHeight(25)
	f.Text:SetPoint("TOPLEFT", f, "TOPLEFT")
	f.Text:SetJustifyH("CENTER")
	f.Text:SetJustifyV("MIDDLE")
	f.Text:SetText(text)
	
	f:Hide()

	return f
end